#ifndef GAW_H
#define GAW_H

#define MSG_DEBUG 1

#include <rcfile.h>
#include <gawload.h>
#include <msglog.h>


#define AW_MSG_OK 0
#define AW_MSG_FATAL 1


#define AW_NX_MBTN 3
#define AW_NY_MBTN 3

enum _ConversionMode {
   CNV_NORMAL,
   CNV_SCIENTIFIC,
};

/*
 * state of mouse for drag operations
 */
enum _MouseState {
   M_NONE,
      M_CURSOR_DRAG,
      M_SELRANGE_ARMED,
      M_SELRANGE_ACTIVE
};

typedef struct _VwWpInfo VwWpInfo;
typedef struct _DnDData DnDData;
typedef struct _MeasureBtn MeasureBtn;
typedef struct _WavePanel WavePanel;
typedef struct _UserData UserData;
typedef struct _AWCursor AWCursor;
typedef struct _MeasureUnit MeasureUnit;
typedef struct _VisibleWave VisibleWave;
typedef struct _SelRange SelRange;
typedef struct _DataFile DataFile;
typedef enum _SelRangeType SelRangeType;
typedef enum _MouseState MouseState;

extern UserData *userData; 

/*
 * Global structures
 */

/*
 * structure sent as drag-and-drop-data for selecting waveforms.
 */
struct _DnDData {
   int magic;
   WaveVar *dv;
   gpointer data;     /* more data */
};


enum _SelRangeType {
   SR_X = 1,
      SR_Y,
      SR_XY
};


/*
 * structure to transmit data between objet
 */
struct _VwWpInfo {
  WavePanel *wp;
  VisibleWave *vw;
};

/*
 * state related to selecting ranges/regions in X, Y, and XY
 * pixel space of a wavepanel.
 */

struct _SelRange {
   int drawn;
   SelRangeType type;
   WavePanel *wp;
   GdkGC *gc;
   GdkColor gdk_color;
   int y1;
   int y2;
   int x1;
   int x2;
   int x1_root;
   int y1_root;
};

/*
 * Structure to hold data for a single loaded waveform file.
 */

struct _DataFile {
   UserData  *ud;   /* back pointer to userData */
   WaveFile *wf;
   gchar *filename;  
   gchar *label;           /* label for nsert action */  
   GtkWidget *wlist_win;   /* window with scrolling variable list */
   GtkWidget *wlist_vbox;  /* vertical container */
   GtkUIManager *ui;       /* ui manager for variable list window */
   GtkWidget *wlist_box;   /* scrolled box containing DnD variable items */
   int ftag;        /* short tag used to help identify which file is which */
   int ndv;
//   GSList *wvhl;    /* temp variable */
   GdkPixmap *drag_icon;
   GdkPixmap *drag_mask;
};

struct _MeasureBtn {
   AWCursor *csp;      /* pointer to associate cursor */
   int measurefunc;
   WaveVar *var;           // note: might be NULL
   
   GtkWidget *button;
   GtkWidget *label;
};

/*
 * VisibleWave -- a waveform shown in a panel.
 */

struct _VisibleWave {
   int valid;      /* 1 if valid, 0 if awaiting deletion */
   WaveVar *var;
   DataFile *adf;
   WavePanel *wp;
   char *varname;  /* the variable name from the file */
   int colorn;
   GdkGC *gc;
   GdkColor color;       /* color for the wave */
   GtkWidget *button;
   GtkWidget *label;
   MeasureBtn *mbtn[3];
   VwWpInfo *vwwp;       /* pointer to structure used to transmit data */
};

/*
 * Cursor - structure describing a vertical bar cursor
 */
struct _AWCursor {
   GtkWidget *button;  /* display x measure button */
   GtkWidget *label;   /* display x measure label  */
   gchar *color_name;  /* name for finding color from rc */
   int shown;          /* vertical bar cursor */
   int x;              /* x pixel value */
   double xval;        /* data corresponding x val */
   GdkGC *cursor_gc;   /* cursor gdk gc */
};

/*
 * WavePanel -- describes a single panel containing zero or more waveforms.
 */
struct _WavePanel {
   UserData *ud;           /* pointer to application data */
   GtkWidget *lmvbox;      /* the container for the left part */
   GtkWidget *drawing;     /* DrawingArea for waveforms */
   GdkPixmap *pixmap;
   GdkGC *pg_gc;           /* gc for graticule */
   GdkColor pg_color;      /* color for graticule */
   gboolean valid;         /* 1 if valid, 0 if awaiting deletion */
   gboolean selected;
   
   GList *vwlist;  /* list of VisibleWaves shown in this panel. */
   
   GtkWidget *lmtable;     /* gtk table for left side */
   GtkWidget *label_min;   /* label for min value */
   GtkWidget *label_max;   /* label for max value */
   GtkWidget *label_ylog;  /* label for log scale max value */
   GtkWidget *min_hbox;    /* hbox for label ymin */
   GtkWidget *max_hbox;    /* hbox for label ymax */
   int req_height;         /* requested height */
   int width;              /* panel actual size */
   int height;             /* panel actual size */

   double min_xval; /* min/max data x/y values over whole vwlist */        
   double max_xval;
   double min_yval;
   double max_yval;
   /*
    * starting and ending drawn x-value (independent var),
    * copied from corresponding wtable values when we scroll/zoom,
    * later we may allow individual panels to be "locked" 
    * from global scroll/zoom or otherwise controlled independently
    */
   double start_xval;      
   double end_xval;
   /*
    * ditto for the y-value dimension; start_yval is the bottom. 
    * invariant: if man_yzoom is false these are
    * the same as min_yval and max_yval.
    */
   double start_yval;
   double end_yval;
   int man_yzoom;
   int nextcolor;  /* color to use for next added waveform */
   int logy;       /* Y axis scaling: 0=linear 1=log base 10 */
   int showGrid;  /* show grid in panel */
};

struct _UserData {
   gchar *prog;
   gchar *mainName;
   GtkWidget *window;  /* top level window */
   GtkWidget *dialog_window;
   GtkWidget *mainBox;  /* main container for app */
   GtkWidget *toolBar; /* tool bar */
   GtkUIManager *ui;     /* ui manager */
   GtkActionGroup *vlactions; /* action group for variable list */
   GtkWidget *statusbar;  /* bottom status bar */
   gchar *filename;
   gint  reqpanels;
   
   AWCursor **cursors;  /* 3 cursor storage pointer */
   
   GtkWidget *table;   /* panel container table */

   GtkWidget *xlabel_box;
   GtkWidget *xscrollbar;
   GtkAdjustment *xadj;
   gint  npanels;
   WavePanel **panels; /* pointer table on panels */
   WavePanel *selected_panel; /* selected panel */
   
   GtkWidget *win_xlabel_left;      
   GtkWidget *win_xlabel_right;      
   GtkWidget *win_xlabel_log;
   GtkTooltips *tooltips; 

   double min_xval;        /* minimum and maximum data x values, */
   double max_xval;        /* over all panels */
   double start_xval;      /* starting drawn x-value (independent var) */
   double end_xval;        /* ending drawn x-value */
   
   MouseState mouseState;
   gint drag_button;      /* which button was dragged   */
   int button_down;
   SelRange *srange;       /* structure to store selected range */
   
//   int logx;               /* X axis scaling: 0=linear 1=log base 10 */
   
   GdkGC *hl_gc;
   int suppress_redraw;
   int NWColors ;        /* # of wavecolorN styles expected in the .gtkrc */
   
   GList *all_measure_buttons; /* measure buttons list */
   GList *wdata_list;     /* dataset list */
   
   GSList *imgFormats;   /* list of writtable img formats */
   const gchar *imgFmt;  /* selected img format */
};

/*
 * Global variables
 */

struct st_file_selection {
   int op_flag;
   char *filename;
   struct st_file_format *pff;
};



/*
 * Global functions
 */
void aw_dialog_show ( int type, const char *text);
void aw_connect_proxy (GtkUIManager *ui, GtkAction *action, GtkWidget *proxy,
	       GtkWidget    *statusbar);
void aw_vl_menu_item_add(  UserData *ud, DataFile *wdata);
GtkUIManager *aw_vl_create_ui_manager ( DataFile *wdata );
void aw_update_toggle_state(GtkUIManager *ui, const gchar *path, gboolean is_active);

char *val2str(double val, int mode);
double str2val(char *s);

int val2x(WavePanel *wp, double val, int logs);
int val2y(WavePanel *wp, double val);
double x2val(WavePanel *wp, int x, int logs);
double y2val(WavePanel *wp, int y);

gint am_cmd_zoom_absolute(UserData *ud, double start, double end, int logs );
void am_wavepanel_update_data(WavePanel *wp);
void am_wavetable_update_data(UserData *ud);
void am_gtk_table_move_row(GtkTable *table, int from, int to);
void am_gtk_table_insert_row(GtkWidget *widget, int row);
void am_gtk_table_delete_row(GtkWidget *widget, int row);
void am_gtk_table_rotate_rows(GtkWidget *widget, int from, int to);
int am_gtk_table_get_child_row(GtkWidget *widget, GtkWidget *child);

void ap_create_win_app (UserData *ud);
void ap_add_panel_line(UserData *ud, WavePanel *ppos);
void ap_remove_panel_line(UserData *ud, WavePanel *ppos);
DataFile *ap_load_wave_file (UserData *ud, DataFile *wdata);
void ap_reload_wave_file (DataFile *wdata);
void ap_var_to_panel(WavePanel *wp, WaveVar *dv);
gboolean ap_draw_wavepanel (GtkWidget *widget, GdkEventExpose *event, gpointer data );
void ap_remove_all_wave_selected( UserData *ud);
void ap_panel_full_redraw(WavePanel *wp);
void ap_remove_wave_from_panel(WavePanel *wp, VisibleWave *vw);
void ap_all_panel_redraw(UserData *ud);
void ap_remove_all_wave_if_panel(WavePanel *wp);
void ap_panel_redraw(WavePanel *wp);
void ap_set_all_wp_cursors(UserData *ud, int cnum);
void ap_widget_show(GtkWidget *w,  gboolean show);
gint ap_find_panel_pos(UserData *ud, WavePanel *ppos);
void ap_mbtn_update_all(UserData *ud);
void ap_create_list_win (UserData *ud,  DataFile *wdata);
void ap_delete_wave_file(GtkWidget *w, DataFile *wdata);
void ap_grid_color_allocate (WavePanel *wp, gint use_default);
void ap_wave_color_allocate (VisibleWave *vw, gint use_default);
void ap_dnd_target_event (GtkWidget *widget, GdkDragContext *context,
		     gint  x, gint y,
		     GtkSelectionData   *data,
		     guint info, guint time, gpointer d);
void ap_set_drag_dest(GtkWidget *widget, WavePanel *wp);

void gaw_show_about_dialog (GtkWidget *parent);
#endif /* GAW_H */
