/*
 * rcfile.c - configuration definition functions
 * 
 * include LICENSE
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <pwd.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <msglog.h>

int prog_debug = 0;

#define SEPCHR '/'

/* type de variable de configuration */
enum _ConfTypeInfo {
   TBOOL = 1,
      TDECIMAL,
      THEXA,
      TCHAINE,
      TLIBELLE,
      TCOMMENT,
      TPOINTER,
      TLISTE       /* string with elements separted with ',' */
};

#define TTABLE    256

#define SIZENAME 100
typedef struct _UserPrefsTable UserPrefsTable;
typedef struct _ParsedParams ParsedParams;

struct _UserPrefsTable {
   char *ptk ;                 /* ptr sur le nom externe : TOKEN */
   char *pvar ;                /* ptr sur le nom variable interne */
   int type ;                  /* type entree                    */
   int v_min ;                 /* valeur min                     */
   int v_max ;                 /* valeur max ou longueur         */
   char *pcomment ;            /* ptr sur un commentaire         */
} ;

struct _ParsedParams {
   char *ptk ;                 /* ptr sur le nom externe : TOKEN */
   char *pstr ;                /* ptr sur le nom interne chaine  */
   int type ;                  /* type entree                    */
   char *pcomment ;            /* ptr sur un commentaire         */
   int idx;                    /* index pour la gestion des tables */
   int ln;                     /* line number */
   int n;                      /* list member count */
} ;

/*************************************************************/
 
#define THIS_IS_RCFILE
#include <rcfile.h>

int sizetki = sizeof(key_tk) / sizeof( key_tk[0] ) ;

/*************************************************************/ 

/* local prototypes */
int rc_read_config(char * pfil, UserPrefsTable *ptk, int sizetki );
int rc_ana_config(char * pbuf, ParsedParams *ptk );
int set_var(UserPrefsTable * ptki, ParsedParams *ptk);
void rm_blank(char * p);
char *home_dir (void);

void TO_UPPER(char * p)
{
   for (  ; *p ; p++ ) {
      if ( islower(*p) )
	*p = toupper(*p) ;
   }
}


/*
 * Return the path to the user's .xxxrc.  This is either the value of
 * `XXXRC' environment variable, or `$HOME/.xxxrc'.
 *
 * If the `XXXRC' variable exists but the file does not exist, the
 * function will exit().
 *  if  `XXXRC' env var
 *     return its value
 * 
 *  if dir is given use :
 *    if dir == "." use 
 *      ./XXXrc
 *    else
 *      $HOME/.XXX/XXXrc
 *  else
 *     $HOME/.XXXrc
 * 
 */
char *rc_file_name (char *prog, char *dir)
{
   char *env, *home;
   char *file = NULL;
   char rc_name[64];
   char rc_env[64];
   struct stat st ;
   int len;

   /*
    * Override defaults with your own, if one exists.
    */
   sprintf(rc_name, "%src", prog);
   sprintf(rc_env, "%src", prog);
   TO_UPPER(rc_env);


   /* Try the environment.  */
   env = getenv (rc_env);
   if (env && *env) {
      if (stat(env, &st) < 0 ) {
	 msg_fatal("%s: %s: %s.", prog, env, strerror (errno));
      }
      return strdup(env);
    }

  /* If that failed, try $HOME/.xxxrc.  */
   home = home_dir();
   if ( ! home) {
      msg_error("%s: Can't find home.", prog );
      /* using local file as rc */
      return strdup(rc_name);
   }
   len = strlen(home) + 1 + strlen(rc_name) + 1;
   if ( dir && *dir ) {
      if ( strcmp(dir, ".") == 0 ){
	 /* local file in current directory */
	 return strdup(rc_name);
      }
      len += strlen(dir) + 2 ; 
      file = (char *) malloc(len);
      sprintf (file, "%s/.%s", home, dir);
      if (stat(file, &st) < 0 ) {
	 mkdir(file, 0755);
      }
      sprintf(&file[strlen(file)], "/%s", rc_name);
   } else {
      file = (char *) malloc(len);
      sprintf (file, "%s/.%s", home, rc_name);
   }
   free(home);
   return file;
}


/*
 * Return the user's home directory (strdup-ed), or NULL if none is
 * found.
 */

char *home_dir (void)
{
   char *home = getenv ("HOME");

   if (!home) {
#ifndef WINDOWS
      /*
       * If HOME is not defined, try getting it from the password
       * file.
       */
      struct passwd *pwd = getpwuid (getuid ());
      if (!pwd || !pwd->pw_dir)
	return NULL;
      home = pwd->pw_dir;
#else  /* WINDOWS */
      home = "C:\\";
      home = ws_mypath ();
      /*
       * Maybe I should grab home_dir from registry, but the best
       * that I could get from there is user's Start menu.  It sucks! 
       */
#endif /* WINDOWS */
   }

  return home ? strdup(home) : NULL;
}

char linebuf[BUFSIZ] ;

/*
 * Configuration des tables a partir du fichier pfil
 *     lecture une ligne du fichier de configuration 
 *     suppression du commentaire s'il existe
 *     si valeur est blanc ou si pas de signe = 
 *       alors valeur par defaut
 */


int
rc_read_config(char * pfil, UserPrefsTable *ptk, int sizetki )
{
   FILE *fd ;
   int ret ;
   int i ;
   int found ;
   UserPrefsTable *ptki ;
   ParsedParams tk ;
	
   ret = 0 ;
   if((fd = fopen(pfil,"r")) == NULL) {
      msg_fatal("Can't open file '%s': %s\n", pfil, strerror (errno));
   }
   tk.ln = 0;
   while(fgets(linebuf,sizeof(linebuf),fd) != NULL) {
      tk.ln++;
      /* fprintf(stderr, "%s", linebuf );*/
      linebuf[strlen(linebuf)-1] = '\0' ;
		
      if ( rc_ana_config(linebuf, &tk ) == 0 ) {
	 /* la ligne ne contient pas de token */
	 continue ;
      }
			
      
      /* recherche du nom dans table des entrees */
      ptki = ptk ;
      found = 0 ;
      for(i = 0; i < sizetki ;i++){ 
	 if (strcmp(tk.ptk, ptki->ptk) == 0) {
	    found++ ;
	    break ;
	 }
	 ptki++ ;
      }
		
      if (!found) {
	 /* le nom n'est trouve dans les tables */
	 continue ;
      }
      
      ret += set_var(ptki, &tk ) ;
   }
   fclose(fd) ;
   return(ret) ;
}

char *skipchr(char *s, char *accept)
{
   size_t l;
   l = strspn(s, accept);
   s += l;
   if ( *s ){
      return s;
   }
   return NULL;
}

char *firstchr(char *s, char *reject)
{
   size_t l;
   
   l = strcspn(s, reject);
   s += l;
   if ( *s ){
      return s;
   }
   return NULL;
}

/*
 * analyse une ligne du fichier de config
 * et remplit les champs la structure tki
 *   return 0 si pas "=" dans la ligne
 */
int
rc_ana_config(char * pbuf, ParsedParams *ptk )
{
   char *p ;
   char *p1 ;
   char *p2 ;
   char *p3 ;
   char *p4 ;
	
   /* suppression commentaire */

   ptk->pcomment = 0 ;
   ptk->n = 0 ;
   if ((p1 = strstr(pbuf,"/*")) != 0) {
      *(p1 - 1) = '\0' ;
      p1 += 2 ;
      if (( p1 = skipchr(p1," \t")) != 0) {
	 if ((p = strstr(p1,"*/")) != 0) {
	    p-- ;
	    rm_blank(p) ;
	 }
      }
      ptk->pcomment = p1 ;
   }
			
   /* ignorer les blancs au debut du nom */
   if ((p1 = skipchr(pbuf," \t")) == 0) {
      /* si ligne vide on saute */ 
      return(0) ;
   }
		
   /* recherche = s'il n'existe pas on saute */
   if ((p2 = firstchr(p1,"=")) == 0) {
      return(0) ;
   }
   *p2++ = '\0' ;
		
   /* p2 pointe la valeur affectee a la variable */
   if ((p2 = skipchr(p2," \t")) == 0) {
      /* si valeur vide on saute */ 
      p2 = "" ;
   } 	

   /* si le premier caractere est '"' rechercher le " de fin */
   if ( *p2 == '"' ) {
      p2++ ;
      p3 = p2 ;
      while ((p3 = firstchr(p3, "\"")) != 0) {
	 if (*(p3-1) == '\\' ) {
	    p3++ ;
	 } else {
	    *p3 = '\0' ;
	    break ;
	 }
      }
      ptk->type = TLIBELLE ;
   } else {
      /* rechercher element de liste */
      p4 = p3 = p2 ;
      while ((p3 = firstchr(p3, ",")) != 0) {
	 ptk->n++ ;
	 p4 = ++p3;   /* save last element */
      }
      if (  ptk->n ) {
	 /* ignorer les blancs en debut du denier element */
	 p4 = skipchr(p4," \t");
      }
      /* ignorer les blancs a la fin de la valeur de la variable */
      if ((p3 = firstchr(p4," \t")) != 0) {
	 *p3 = '\0' ;
      }
      ptk->type = TCHAINE ;
   }

   ptk->idx = -1;
   /* rechercher un index eventuel dans le nom de variable */
   if ((p4 = firstchr(p1,"[")) != 0) {
      *p4 = '\0' ;
      p4++;
      /* ignorer les blancs avant l'index */
      if ((p3 = skipchr(p4," \t")) == 0) {
	 p3 = p4;
      }
      /* ignorer les blancs apres l'index */
      if ((p4 = skipchr(p3," \t]")) != 0) {
	 *p4 = '\0';
      }
      if ( sscanf( p3, "%d", &ptk->idx) != 1) {
	 msg_warning("Syntax error at line %d", ptk->ln) ;
	 return(0);
      }
      ptk->type |= TTABLE ;
   }
	
   /* ignorer les blancs a la fin du nom de la variable */
   if ((p3 = firstchr(p1," \t")) != 0) {
      *p3 = '\0' ;
   }
	
   ptk->ptk = p1 ;
   ptk->pstr = p2 ;
   
   return(1) ;
}

void
rm_blank(char * p)
{
   while( *p && ( *p == ' ' || *p == '\t' ) ) {
      *p = '\0' ;
      p-- ;
   }
}

void rmliste(char **pl)
{
   char **pptr = pl;
   char *ptr;
   
   while ( (ptr = *pptr++) ){
      free(ptr);
   }
   free(pl);
}

char **mkliste( ParsedParams *ptk)
{
   char *p, *p1, *p2, *p3;
   char **ptr;
   char **pptr;
   int i;
   
   p = ptk->pstr;
   i = ++ptk->n;
   pptr = ptr = (char **) malloc( (i +1 ) * sizeof(char *) );
   if ( ptr == NULL ){
      msg_fatal("malloc return NULL");
   }
   p1 = p;
   while ( i > 0 ){
      /* ignorer les blancs au debut */
      if (  (p3 = skipchr(p1, " \t")) != 0) {
	  p1 = p3;
      }
      /* chercher le separateur */
      if (  (p2 = firstchr(p1, ",")) != 0) {
	 *p2++ = 0;
      }
      /* ignorer les blancs a la fin */
      if (  (p3 = firstchr(p1, " \t")) != 0) {
	 *p3 = 0;
      }
      *pptr++ = strdup(p1);
      p1 = p2;
      i--;
   }
   *pptr = NULL;
   return ptr;
}

int
set_var(UserPrefsTable * ptki, ParsedParams *ptk)
{
   char *pvar ;
   int *pi;
   char *p;
   int val ;

   pvar = ptki->pvar ;
   pi = (int *) pvar;
   p = ptk->pstr;
   
   switch(ptki->type) {
    case TBOOL :
      if (*p == '\0') {
	return(0) ; /* conserver valeur default */
      }
      if ( sscanf(p, "%d" , &val) == 1) {
	 if (val != 0 ) {
	   val = 1 ;
	 }
	 *pi = val ;
	 return(0) ;
      }
      break ;
    case TDECIMAL :
      if (*p == '\0')
	return(0) ; /* conserver valeur default */
      if ( sscanf(p, "%d" , &val) == 1) {
	 if (val >= ptki->v_min && val <= ptki->v_max ) {
	    *pi = val ;
	    return(0) ;
	 }
      }
      break ;
    case THEXA :
      if (*p == '\0')
	return(0) ; /* conserver valeur default */
      if ( sscanf( p, "%x" , &val) == 1) {
	 if (val >= ptki->v_min && val < ptki->v_max ) {
	    *pi = val ;
	    return(0) ;
	 }
      }
      break ;
    case TLIBELLE :
    case TCHAINE :
      if ( strlen( p ) < ptki->v_max ) {
	 strcpy( pvar, p ) ;
	 return(0) ;
      }
      break ;
    case TPOINTER :
      {
	 char **pv = (char **) pvar;
	 if ( *pv ) {
	    free( *pv);
	 }
	 *pv = strdup( p ) ;
      }
      return(0) ;
    case TLISTE :
      {
	 char **pv = (char **) pvar;
	 if ( *pv ) {
	    rmliste( (char **) *pv);
	 }
	 *pv = (char *) mkliste( ptk ) ;
      }
      return(0) ;
   }
   msg_warning("Variable '%s' ignored. Val '%s' at line %d",
	       ptki->ptk, p, ptk->ln) ;
   return(1) ;
}

/*
 * sort une ligne de fichier de config dans un fichier
 */
int
rc_out_config( FILE * fd, UserPrefsTable *ptki )
{
   int ll ;
   int ll1 ;
   char *p ;
   char **ptr ;
   char lbuf[BUFSIZ] ;
   char *pvar;
   int *pi;
   
   pvar = ptki->pvar;
   pi = (int *) pvar;
   switch(ptki->type) {
    case TBOOL :
    case TDECIMAL :
      sprintf(lbuf, "%s = %d ",
	      ptki->ptk, *pi ) ;
      break ;
    case THEXA :
      sprintf(lbuf, "%s = %x ",
	      ptki->ptk, *pi ) ;
      break ;
    case TCHAINE :
      sprintf(lbuf, "%s = %s ",
	      ptki->ptk, pvar ) ;
      break ;
    case TPOINTER :
      {
	 char **pv = (char **) pvar;
	 p = *pv ;
	 if ( p == NULL) {
	    p = "";
	 }
	 sprintf(lbuf, "%s = %s ",  ptki->ptk, p ) ;
      }
      break ;
    case TLIBELLE :
      sprintf(lbuf, "%s = \"%s\" ", ptki->ptk, pvar ) ;
      break ;
    case TCOMMENT :
      fprintf(fd, "\n  /* %s */\n\n", ptki->pcomment);
      ptki++ ;
      return(0) ;
    case TLISTE :
      {
	 char **pv = (char **) pvar;
	 ptr = (char **) *pv ;
	 sprintf(lbuf, "%s = ", ptki->ptk ) ;
	 if ( ptr == NULL) {
	    strcat(lbuf, " ");
	    break;
	 }
	 while ( *ptr ) {
	    strcat(lbuf, *ptr);
	    ptr++;
	    if ( *ptr ){
	       strcat(lbuf, ", ");
	    }
	 }
	 break ;
      }
   }
	
   /*
    * si long ligne + lg comment > 80
    *  imprimer le commentaire avant
    *  sinon imprimer commentaire en fin de ligne
    */
   
   ll = strlen(lbuf) ;
   ll1 = strlen(ptki->pcomment) + 3 ;
   if ( ll + ll1 < 80 ) {
      fprintf(fd, "%s", lbuf);
      for ( ; ll < 40 ; ll++) {
	 fprintf(fd, " ");
      }
      fprintf(fd, "/* %s", ptki->pcomment);
      ll += ll1 ;
      for ( ; ll < 76 ; ll++) {
	fprintf(fd, " ");
      }
      fprintf(fd, " */\n");
   } else {
      fprintf(fd, "    /* %s", ptki->pcomment);
      fprintf(fd, " */\n");
      fprintf(fd, "%s", lbuf);
      fprintf(fd, "\n");
   }
   return(0) ;
}

int rc_save_config_file (char * file)
{
   int sz ;
   int i ;
   UserPrefsTable *ptki ;
   FILE *fd ;
   
   if((fd = fopen(file, "w")) == NULL) {
      msg_fatal("Can't open file '%s': %s\n", file, strerror (errno));
   }

   ptki = key_tk ;
   sz = sizetki ;
   for ( i = 0 ; i < sz ; i++ ) {
      rc_out_config( fd, ptki ) ;
      ptki++ ;
   }

   fclose(fd) ;
   return(0) ;
}

int rc_save_config(char *prog, char *dir)
{
   char *file ;
   int ret;
   
   file = rc_file_name (prog, dir);
   if ( file == NULL ) {
      return(-1) ;
   }
   ret = rc_save_config_file(file);
   free (file);
   return ret;
}


/*
 *   use filename to read config file
 */
int rc_init_config_file( char *filename)
{
   int ret;
   struct stat st ;

   rc_init_defaults();
   if (stat(filename, &st) < 0 ) {
      ret = rc_save_config_file(filename);
      msg_info("created config file '%s'",  filename );
      return(ret);
   }
   return rc_read_config(filename, key_tk, sizetki);
}

/*
 * find rc file from prog name 
 *   or use filename if given
 */
int rc_init_config(char *prog, char *dir)
{
   char *file;
   int ret;
   
   file = rc_file_name (prog, dir);
   if ( file == NULL ) {
      return(-1) ;
   }

   ret = rc_init_config_file(file);
   free (file);
   return(ret);
}
