#ifndef RCFILE_H
#define RCFILE_H
 
/*
 * rcfile.h - program configuration definition
 * 
 * include LICENSE
 */

/*
 * WARNING : This file use the same name for each project
 *           but its contents is prokect dependant
 */

typedef struct _UserPrefs {
   int  npanels;
   int  min_win_width;
   int  min_win_height;
   int  gridXSpacing;
   int  gridYSpacing;
   int  panelHeight;
   int  panelWidth;
   int  lmtableWidth;
   int  showGrid;
   int  showXLabels;
   int  showYLabels;
   int  setLogX;
   int  setLogY;
   int  showYDiff;
   char *imgFmt;
   char *imgBaseName;
   int  jpegQuality;
   int  pngCompression;
} UserPrefs;

extern UserPrefs userPrefs[];

#ifdef THIS_IS_RCFILE

/*
 * userPrefs must be global - don't malloc it
 */
UserPrefs userPrefs[1];

 
UserPrefsTable key_tk[] = {
     { "", 0 , TCOMMENT, 0, 0,
           "User Preferences"},
     { "Prog_Debug", (char *) &prog_debug , TDECIMAL, 0, 4,
           "Pgrm Debug  0=NO , 1=Yes"      },
     { "Panel_Number", (char *) &userPrefs->npanels, TDECIMAL, 0, 10,
           "Number of panels to be displayed by default" },
     { "min_win_width", (char *) &userPrefs->min_win_width, TDECIMAL, 40, 1024,
	  "Window minimum width"},
     { "grid_x_spacing", (char *) &userPrefs->gridXSpacing, TDECIMAL, 0, 100,
	  "grid X line spacing"},
     { "grid_y_spacing", (char *) &userPrefs->gridYSpacing, TDECIMAL, 0, 100,
	  "grid Y line spacing"},
     { "min_win_height", (char *) &userPrefs->min_win_height, TDECIMAL, 40, 1024,
	  "Window minimum height"},
     { "panel_height", (char *) &userPrefs->panelHeight, TDECIMAL, 40, 1024,
	  "nominal height for panel"},
     { "panel_width", (char *) &userPrefs->panelWidth, TDECIMAL, 40, 1024,
	  "nominal width for panel"},
     { "lmtable_width", (char *) &userPrefs->lmtableWidth, TDECIMAL, 40, 1024,
	  "nominal width for buttons and masurements table"},
     { "show_grid", (char *) &userPrefs->showGrid, TBOOL, 0, 1,
	  "Should we show yhe grid in panel"},
     { "show_xlabels", (char *) &userPrefs->showXLabels, TBOOL, 0, 1,
	  "Should we show X labels"},
     { "show_ylabels", (char *) &userPrefs->showYLabels, TBOOL, 0, 1,
	  "Should we show Y labels"},
     { "set_logX", (char *) &userPrefs->setLogX, TBOOL, 0, 1,
	  "Should we set log X scale"},
     { "set_logY", (char *) &userPrefs->setLogY, TBOOL, 0, 1,
	  "Should we set log Y scale"},
     { "show_Ydiff", (char *) &userPrefs->showYDiff, TBOOL, 0, 1,
	  "Should we show Y diff measure buttons"},
     { "img_fmt", (char *) &userPrefs->imgFmt, TPOINTER, 0, SIZENAME,
           "image default format: jpeg, png, tiff,..." },
     { "img_name", (char *) &userPrefs->imgBaseName, TPOINTER, 0, SIZENAME,
           "Base name for saving image" },
     { "jpeg_quality", (char *) &userPrefs->jpegQuality, TDECIMAL, 0, 100,
	  "jpeg quality for saving image" },
     { "png_compression", (char *) &userPrefs->pngCompression, TDECIMAL, 0, 9,
	  "png compression for saving image" },
 } ;

void rc_init_defaults(void)
{
   memset(userPrefs, 0, sizeof(UserPrefs));
   userPrefs->npanels = 2;
   userPrefs->min_win_width = 80;
   userPrefs->min_win_height = 50;
   userPrefs->gridXSpacing = 20;
   userPrefs->gridYSpacing = 20;
   userPrefs->panelHeight = 100;
   userPrefs->panelWidth = 400;
   userPrefs->lmtableWidth = 280;
   userPrefs->showXLabels = 1;
   userPrefs->showYLabels = 1;
   userPrefs->imgFmt = strdup("png");
   userPrefs->imgBaseName = strdup("panel");
   userPrefs->jpegQuality = 100;
}

#endif /* THIS_IS_RCFILE */

extern int prog_debug;

int rc_init_config(char *prog, char *dir);
int rc_init_config_file( char *filename);
int rc_save_config(char *prog, char *dir);
int rc_save_config_file(char *filename);
char *rc_file_name (char *prog, char *dir);

#endif /* RCFILE_H */
